// src/signature-pad.anatomy.ts
import { createAnatomy } from "@zag-js/anatomy";
var anatomy = createAnatomy("signature-pad").parts(
  "root",
  "control",
  "segment",
  "segmentPath",
  "guide",
  "clearTrigger",
  "label"
);
var parts = anatomy.build();

// src/signature-pad.connect.ts
import { getNativeEvent, getRelativePoint, isLeftClick, isModifierKey } from "@zag-js/dom-event";
import { dataAttr, getEventTarget } from "@zag-js/dom-query";

// src/signature-pad.dom.ts
import { createScope, query } from "@zag-js/dom-query";
var dom = createScope({
  getRootId: (ctx) => `signature-${ctx.id}`,
  getControlId: (ctx) => `signature-control-${ctx.id}`,
  getHiddenInputId: (ctx) => `signature-input-${ctx.id}`,
  getControlEl: (ctx) => dom.getById(ctx, dom.getControlId(ctx)),
  getSegmentEl: (ctx) => query(dom.getControlEl(ctx), "[data-part=segment]"),
  getHiddenInputEl: (ctx) => dom.getById(ctx, dom.getHiddenInputId(ctx)),
  getDataUrl: (ctx, options) => {
    const { type, quality = 0.92 } = options;
    if (ctx.isEmpty) {
      return Promise.resolve("");
    }
    const svg = dom.getSegmentEl(ctx);
    if (!svg) {
      throw new Error("Could not find the svg element.");
    }
    const win = dom.getWin(ctx);
    const doc = win.document;
    const serializer = new win.XMLSerializer();
    const source = '<?xml version="1.0" standalone="no"?>\r\n' + serializer.serializeToString(svg);
    const svgString = "data:image/svg+xml;charset=utf-8," + encodeURIComponent(source);
    if (type === "image/svg+xml") {
      return Promise.resolve(svgString);
    }
    const svgBounds = svg.getBoundingClientRect();
    const dpr = win.devicePixelRatio || 1;
    const canvas = doc.createElement("canvas");
    const image = new win.Image();
    image.src = svgString;
    canvas.width = svgBounds.width * dpr;
    canvas.height = svgBounds.height * dpr;
    const context = canvas.getContext("2d");
    context.scale(dpr, dpr);
    return new Promise((resolve) => {
      image.onload = () => {
        context.drawImage(image, 0, 0);
        resolve(canvas.toDataURL(type, quality));
      };
    });
  }
});

// src/signature-pad.connect.ts
function connect(state, send, normalize) {
  const isDrawing = state.matches("drawing");
  const isEmpty = state.context.isEmpty;
  const isInteractive = state.context.isInteractive;
  const isDisabled = !!state.context.disabled;
  return {
    isEmpty,
    isDrawing,
    currentPath: state.context.currentPath,
    paths: state.context.paths,
    clear() {
      send({ type: "CLEAR" });
    },
    getDataUrl(type, quality) {
      return dom.getDataUrl(state.context, { type, quality });
    },
    labelProps: normalize.element({
      ...parts.label.attrs,
      "data-disabled": dataAttr(isDisabled),
      htmlFor: dom.getControlId(state.context)
    }),
    rootProps: normalize.element({
      ...parts.root.attrs,
      "data-disabled": dataAttr(isDisabled),
      id: dom.getRootId(state.context)
    }),
    controlProps: normalize.element({
      ...parts.control.attrs,
      tabIndex: isDisabled ? void 0 : 0,
      id: dom.getControlId(state.context),
      "aria-label": "Signature Pad",
      "aria-roledescription": "signature pad",
      "aria-disabled": isDisabled,
      "data-disabled": dataAttr(isDisabled),
      onPointerDown(event) {
        const target = getEventTarget(getNativeEvent(event));
        if (target?.closest("[data-part=clear-trigger]"))
          return;
        if (!isLeftClick(event) || isModifierKey(event) || !isInteractive)
          return;
        event.currentTarget.setPointerCapture(event.pointerId);
        const point = { x: event.clientX, y: event.clientY };
        const { offset } = getRelativePoint(point, dom.getControlEl(state.context));
        send({ type: "POINTER_DOWN", point: offset, pressure: event.pressure });
      },
      onPointerUp(event) {
        if (!isInteractive)
          return;
        event.currentTarget.releasePointerCapture(event.pointerId);
      },
      style: {
        position: "relative",
        touchAction: "none",
        userSelect: "none"
      }
    }),
    segmentProps: normalize.svg({
      ...parts.segment.attrs,
      style: {
        position: "absolute",
        top: 0,
        left: 0,
        width: "100%",
        height: "100%",
        pointerEvents: "none",
        fill: state.context.drawing.fill
      }
    }),
    getSegmentPathProps(props2) {
      return normalize.path({
        ...parts.segmentPath.attrs,
        d: props2.path
      });
    },
    guideProps: normalize.element({
      ...parts.guide.attrs,
      "data-disabled": dataAttr(isDisabled)
    }),
    clearTriggerProps: normalize.button({
      ...parts.clearTrigger.attrs,
      type: "button",
      "aria-label": "Clear Signature",
      hidden: !state.context.paths.length || isDrawing,
      disabled: isDisabled,
      onClick() {
        send({ type: "CLEAR" });
      }
    }),
    getHiddenInputProps(props2) {
      return normalize.input({
        type: "text",
        hidden: true,
        disabled: isDisabled,
        name: state.context.name,
        value: props2.value
      });
    }
  };
}

// src/signature-pad.machine.ts
import { createMachine } from "@zag-js/core";
import { getRelativePoint as getRelativePoint2, trackPointerMove } from "@zag-js/dom-event";
import { compact } from "@zag-js/utils";
import getStroke from "perfect-freehand";

// src/get-svg-path.ts
var average = (a, b) => (a + b) / 2;
function getSvgPathFromStroke(points, closed = true) {
  const len = points.length;
  if (len < 4) {
    return "";
  }
  let a = points[0];
  let b = points[1];
  const c = points[2];
  let result = `M${a[0].toFixed(2)},${a[1].toFixed(2)} Q${b[0].toFixed(2)},${b[1].toFixed(2)} ${average(b[0], c[0]).toFixed(2)},${average(
    b[1],
    c[1]
  ).toFixed(2)} T`;
  for (let i = 2, max = len - 1; i < max; i++) {
    a = points[i];
    b = points[i + 1];
    result += `${average(a[0], b[0]).toFixed(2)},${average(a[1], b[1]).toFixed(2)} `;
  }
  if (closed) {
    result += "Z";
  }
  return result;
}

// src/signature-pad.machine.ts
function machine(userContext) {
  const ctx = compact(userContext);
  return createMachine(
    {
      id: "signature-pad",
      initial: "idle",
      context: {
        ...ctx,
        paths: [],
        currentPoints: [],
        currentPath: null,
        drawing: {
          size: 2,
          simulatePressure: false,
          thinning: 0.7,
          smoothing: 0.4,
          streamline: 0.6,
          ...ctx.drawing
        }
      },
      computed: {
        isInteractive: (ctx2) => !(ctx2.disabled || ctx2.readOnly),
        isEmpty: (ctx2) => ctx2.paths.length === 0
      },
      on: {
        CLEAR: {
          actions: ["clearPoints", "invokeOnDrawEnd", "focusCanvasEl"]
        }
      },
      states: {
        idle: {
          on: {
            POINTER_DOWN: {
              target: "drawing",
              actions: ["addPoint"]
            }
          }
        },
        drawing: {
          activities: ["trackPointerMove"],
          on: {
            POINTER_MOVE: {
              actions: ["addPoint", "invokeOnDraw"]
            },
            POINTER_UP: {
              target: "idle",
              actions: ["endStroke", "invokeOnDrawEnd"]
            }
          }
        }
      }
    },
    {
      activities: {
        trackPointerMove(ctx2, _evt, { send }) {
          const doc = dom.getDoc(ctx2);
          return trackPointerMove(doc, {
            onPointerMove({ event, point }) {
              const { offset } = getRelativePoint2(point, dom.getControlEl(ctx2));
              send({ type: "POINTER_MOVE", point: offset, pressure: event.pressure });
            },
            onPointerUp() {
              send({ type: "POINTER_UP" });
            }
          });
        }
      },
      actions: {
        addPoint(ctx2, evt) {
          ctx2.currentPoints.push(evt.point);
          const stroke = getStroke(ctx2.currentPoints, ctx2.drawing);
          ctx2.currentPath = getSvgPathFromStroke(stroke);
        },
        endStroke(ctx2) {
          ctx2.paths.push(ctx2.currentPath);
          ctx2.currentPoints = [];
          ctx2.currentPath = null;
        },
        clearPoints(ctx2) {
          ctx2.currentPoints = [];
          ctx2.paths = [];
        },
        focusCanvasEl(ctx2) {
          queueMicrotask(() => {
            dom.getControlEl(ctx2)?.focus({ preventScroll: true });
          });
        },
        invokeOnDraw(ctx2) {
          ctx2.onDraw?.({
            paths: [...ctx2.paths, ctx2.currentPath]
          });
        },
        invokeOnDrawEnd(ctx2) {
          ctx2.onDrawEnd?.({
            paths: [...ctx2.paths],
            getDataUrl(type, quality = 0.92) {
              return dom.getDataUrl(ctx2, { type, quality });
            }
          });
        }
      }
    }
  );
}

// src/signature-pad.props.ts
import { createProps } from "@zag-js/types";
import { createSplitProps } from "@zag-js/utils";
var props = createProps()([
  "dir",
  "disabled",
  "getRootNode",
  "id",
  "onDraw",
  "onDrawEnd",
  "readOnly",
  "drawing",
  "name"
]);
var splitProps = createSplitProps(props);
export {
  anatomy,
  connect,
  machine,
  props,
  splitProps
};
//# sourceMappingURL=index.mjs.map